﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirplaneWithRadar
{
    internal abstract class AbstractMap
    {
        private IDrawningObject _drawningObject = null;
        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;
        public Bitmap CreateMap(int width, int height, IDrawningObject drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }
        private bool HasCollisionBarrier(float x, float y, float right, float bottom)
        {
            if (x < 0 || y < 0 || right > _width || bottom > _height)
            {
                return false;
            }
            for (float i = x / _size_x; i < right / _size_x; i++)
            {
                for (float j = y / _size_y; j < bottom / _size_y; j++)
                {
                    if (_map[(int)i, (int)j] == _barrier)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
        public Bitmap MoveObject(Direction direction)
        {
            (float left, float top, float right, float bottom) = _drawningObject.GetCurrentPosition();
            float width = right - left, height = bottom - top;
            if (direction == Direction.Up)
            {
                top -= _drawningObject.Step;
            }
            if (direction == Direction.Down)
            {
                top += _drawningObject.Step;
            }
            if (direction == Direction.Left)
            {
                left -= _drawningObject.Step;
            }
            if (direction == Direction.Right)
            {
                left += _drawningObject.Step;
            }
            if (!HasCollisionBarrier(left, top, left+width, top+height))
            {
                _drawningObject.MoveObject(direction);
            }
            return DrawMapWithObject();
        }

        private bool SetObjectOnMap()
        {
            if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 100);
            int y = _random.Next(0, 100);
            _drawningObject.SetObject(x, y, _width, _height);
            (float left, float top, float right, float bottom) = _drawningObject.GetCurrentPosition();
            
            return !HasCollisionBarrier(x, y, right, bottom);
        }
        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (_map[i, j] == _freeRoad)
                    {
                        DrawRoadPart(gr, i, j);
                    }
                    else if (_map[i, j] == _barrier)
                    {
                        DrawBarrierPart(gr, i, j);
                    }
                }

            }
            _drawningObject.DrawningObject(gr);
            return bmp;
        }
        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}