﻿namespace AirplaneWithRadar
{
    /// <summary>
    /// Карта с набром объектов под нее
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <typeparam name="U"></typeparam>
    internal class MapWithSetAirplanesGeneric<T, U>
        where T : class, IDrawningObject
        where U : AbstractMap
    {
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 210;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 90;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetAirplanesGeneric<T> _setAirplanes;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetAirplanesGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setAirplanes = new SetAirplanesGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="airplane"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetAirplanesGeneric<T, U> map, T airplane)
        {
            return map._setAirplanes.Insert(airplane);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetAirplanesGeneric<T, U> map, int position)
        {
            return map._setAirplanes.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawAirplanes(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();
            for (int i = 0; i < _setAirplanes.Count; i++)
            {
                var airplane = _setAirplanes.Get(i);
                if (airplane != null)
                {
                    return _map.CreateMap(_pictureWidth, _pictureHeight, airplane);
                }
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// </summary>
        private void Shaking()
        {
            int j = _setAirplanes.Count - 1;
            for (int i = 0; i < _setAirplanes.Count; i++)
            {
                if (_setAirplanes.Get(i) == null)
                {
                    for (; j > i; j--)
                    {
                        var airplane = _setAirplanes.Get(j);
                        if (airplane != null)
                        {
                            _setAirplanes.Insert(airplane, i);
                            _setAirplanes.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        private void DrawHangar(Graphics g, int x, int y, int width, int height)
        {
            Pen pen = new(Color.Black, 3);
            g.DrawLine(pen, x, y, x + width, y);
            g.DrawLine(pen, x, y, x, y + height);
            g.DrawLine(pen, x, y + height, x + width, y + height);
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {
                    DrawHangar(g, i * _placeSizeWidth, j * _placeSizeHeight , _placeSizeWidth * 3 / 4, _placeSizeHeight - 55);
                }
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawAirplanes(Graphics g)
        {
            int countInLine = _pictureWidth / _placeSizeWidth;
            int maxLeft = (countInLine - 1) * _placeSizeWidth;
            for (int i = 0; i < _setAirplanes.Count; i++)
            {
                var airplane = _setAirplanes.Get(i);
                airplane?.SetObject(maxLeft - i % countInLine * _placeSizeWidth + 5, i / countInLine * _placeSizeHeight + 15 , _pictureWidth, _pictureHeight);
                airplane?.DrawningObject(g);
            }
        }
    }
}